﻿/*
This file is part of Random Multiplier.

Random Multiplier is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version.

Random Multiplier is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
Random Multiplier. If not, see <https://www.gnu.org/licenses/>. 
*/

using HarmonyLib;
using System;
using System.Collections.Generic;
using System.Reflection;

namespace random_mult
{
  // Inspiration from Misc Cheats by Aidanamite

  [HarmonyPatch(typeof(PlayerInventory), "AddItem")]
  static class patch_PlayerInventory_AddItem
  {
    [HarmonyPatch(new Type[] { typeof(string), typeof(int) })]
    [HarmonyPrefix]
    static void Prefix1(ref int amount)
    {
      if (main.should_scale)
        amount *= main.get_rand();
    }

    [HarmonyPatch(new Type[] { typeof(string), typeof(Slot), typeof(int) })]
    [HarmonyPrefix]
    static void Prefix2(ref int amount)
    {
      if (main.should_scale)
        amount *= main.get_rand();
    }

    [HarmonyPatch(new Type[] { typeof(ItemInstance), typeof(bool) })]
    [HarmonyPrefix]
    static void Prefix3(ItemInstance itemInstance)
    {
      if (main.should_scale)
        itemInstance.Amount *= main.get_rand();
    }
  }

  [HarmonyPatch(typeof(Pickup), "AddItemToInventory")]
  static class patch_Pickup_AddItemToInventory
  {
    static void Prefix(PickupItem item)
    {
      var term = item.pickupTerm;
      if (item.isDropped)
        main.should_scale = main.multiply(main.MULTIPLY_DROPPED);
      else if (main.is_harvesting)
        main.should_scale = main.multiply(main.MULTIPLY_HARVESTING);
      else if (term is string)
      {
        if (term.StartsWith("Item/Arrow_"))
          main.should_scale = main.multiply(main.MULTIPLY_ARROWS);
        else
        {
          // Do we really need multiple recipes and blueprints?
          main.should_scale = !term.StartsWith("Item/Recipe_")
            && !term.StartsWith("Item/Blueprint");
        }
      }
      else
        main.should_scale = true;
    }
    static void Postfix() => main.should_scale = false;
  }

  [HarmonyPatch]
  static class patch_multiply_general
  {
    readonly static (Type, string)[] METHODS =
    {
      (typeof(Block_CookingStand), "CollectItem"),
      (typeof(Placeable_Extractor), "HarvestOutput"),
      (typeof(ResourceCollector), "OnHarvest")
    };

    static IEnumerable<MethodBase> TargetMethods()
    {
      foreach (var (type, name) in METHODS)
        yield return AccessTools.Method(type, name);
    }

    static void Prefix() => main.should_scale = true;
    static void Postfix() => main.should_scale = false;
  }

#if false
  // Allowing water purifiers to multiply output seems to be glitchy.
  [HarmonyPatch(typeof(Block_CookingStand_Purifier), "CollectItem")]
  static class patch_Block_CookingStand_Purifier_CollectItem
  {
    static void Prefix() => main.should_scale = false;
    static void Postfix() => main.should_scale = false;
  }
#endif

  [HarmonyPatch]
  static class patch_multiply_cooking
  {
    readonly static (Type, string)[] METHODS =
{
      (typeof(CookingTable), "PickupFood"),
      (typeof(Block_CookingStand), "CollectItem")
    };

    static IEnumerable<MethodBase> TargetMethods()
    {
      foreach (var (type, name) in METHODS)
        yield return AccessTools.Method(type, name);
    }

    static void Prefix()
    {
      main.should_scale = main.multiply(main.MULTIPLY_COOKING);
    }
    static void Postfix() => main.should_scale = false;
  }

  [HarmonyPatch(typeof(CraftingMenu), "CraftItem")]
  static class patch_multiply_crafting
  {
    static void Prefix()
    {
      main.should_scale = main.multiply(main.MULTIPLY_CRAFTING);
    }
    static void Postfix() => main.should_scale = false;
  }

  [HarmonyPatch(typeof(FishingRod), "PullItemsFromSea")]
  static class patch_multiply_fishing
  {
    static void Prefix()
    {
      main.should_scale = main.multiply(main.MULTIPLY_FISHING);
    }
    static void Postfix() => main.should_scale = false;

  }

  [HarmonyPatch]
  static class patch_multiply_havesting
  {
    readonly static (Type, string)[] METHODS =
    {
      (typeof(BeeHive), "HarvestYield"),
      (typeof(BirdsNest), "PickupEgg"),
      (typeof(BirdsNest), "PickupFeather"),
      (typeof(HarvestableTree), "Harvest"),
      (typeof(PlantManager), "HarvestPlants"),
      (typeof(Shark), "CollectYield")
    };

    static IEnumerable<MethodBase> TargetMethods()
    {
      foreach (var (type, name) in METHODS)
        yield return AccessTools.Method(type, name);
    }

    static void Prefix()
    {
      main.is_harvesting = true;
      main.should_scale = main.multiply(main.MULTIPLY_HARVESTING);
    }
    static void Postfix()
    {
      main.is_harvesting = false;
      main.should_scale = false;
    }
  }

  [HarmonyPatch]
  static class patch_multiply_trading
  {
    readonly static (Type, string)[] METHODS =
    {
      (typeof(TradingPost), "PurchaseItem"),
      (typeof(TradingPostUI), "Button_Sell"),
      (typeof(VendingMachine), "PickupVendingHatch")
    };

    static IEnumerable<MethodBase> TargetMethods()
    {
      foreach (var (type, name) in METHODS)
        yield return AccessTools.Method(type, name);
    }

    static void Prefix()
    {
      main.should_scale = main.multiply(main.MULTIPLY_TRADING);
    }
    static void Postfix() => main.should_scale = false;
  }
}
