/*
This file is part of Random Multiplier.

Random Multiplier is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version.

Random Multiplier is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
Random Multiplier. If not, see <https://www.gnu.org/licenses/>. 
*/

using HarmonyLib;
using HMLLibrary;
using RaftModLoader;
using System;
using System.Collections.Generic;
using System.Runtime.CompilerServices;
using System.Text.RegularExpressions;
using Debug = UnityEngine.Debug;

namespace random_mult
{
  public class main : Mod
  {
    enum e_distrubtion
    {
      Log,
      Constant,
      Linear,
      Polynomial,
      Max
    };

    static Harmony harmony;
    public static bool is_harvesting = false;
    static System.Random rand = new System.Random();
    public static bool should_scale = false;

    public const int MULTIPLY_ARROWS = 0;
    public const int MULTIPLY_COOKING = 1;
    public const int MULTIPLY_CRAFTING = 2;
    public const int MULTIPLY_DROPPED = 3;
    public const int MULTIPLY_FISHING = 4;
    public const int MULTIPLY_HARVESTING = 5;
    public const int MULTIPLY_TRADING = 6;
    static Dictionary<string, int> multiplier_names =
      new Dictionary<string, int>
      {
        {"arrows", MULTIPLY_ARROWS},
        {"cooking", MULTIPLY_COOKING},
        {"crafting", MULTIPLY_CRAFTING},
        {"dropped", MULTIPLY_DROPPED},
        {"fishing", MULTIPLY_FISHING},
        {"harvesting", MULTIPLY_HARVESTING},
        {"trading", MULTIPLY_TRADING}
      };

    static int multipliers;
    public static bool multiply(int offset) => (multipliers & 1 << offset) != 0;

    static e_distrubtion distribution = e_distrubtion.Log;
    static float factor = 2;

    [MethodImpl(MethodImplOptions.NoInlining)]
    static string ExtraSettingsAPI_GetInputValue(string var) => null;
    [MethodImpl(MethodImplOptions.NoInlining)]
    static bool ExtraSettingsAPI_GetCheckboxState(string var) => false;
    [MethodImpl(MethodImplOptions.NoInlining)]
    static int ExtraSettingsAPI_GetComboboxSelectedIndex(string var) => -1;

    public void Start()
    {
      harmony = new Harmony("com.deciusmagnus.random_mult");
      harmony.PatchAll();
      Debug.Log("Mod random_mult has been loaded!");
    }

    public void OnModUnload()
    {
      harmony.UnpatchAll(harmony.Id);
      Debug.Log("Mod random_mult has been unloaded!");
    }
    void ExtraSettingsAPI_Load() => load();
    void ExtraSettingsAPI_SettingsClose() => load();

    void load()
    {
      float f;
      int i;

      if (float.TryParse(ExtraSettingsAPI_GetInputValue("factor"),
        out f))
      {
        factor = f;
      }

      i = ExtraSettingsAPI_GetComboboxSelectedIndex("distribution");
      if (0 <= i || i < (int)e_distrubtion.Max)
        distribution = (e_distrubtion)i;

      foreach (var pair in multiplier_names)
      {
        if (ExtraSettingsAPI_GetCheckboxState($"multiply_{pair.Key}"))
          multipliers |= 1 << pair.Value;
        else
          multipliers &= ~(1 << pair.Value);
      }
    }

    public static int get_rand()
    {
      int ret;

      switch (distribution)
      {
      case e_distrubtion.Log:
        ret = (int)Math.Floor(1 - Math.Log(1 - rand.NextDouble(), factor));
        break;
      case e_distrubtion.Constant:
        ret = (int)Math.Round(factor);
        break;
      case e_distrubtion.Linear:
        ret = (int)Math.Round((1 - rand.NextDouble()) * factor);
        break;
      case e_distrubtion.Polynomial:
        ret = (int)Math.Round(Math.Pow(factor, 1 - rand.NextDouble()));
        break;
      default:
        Debug.Log("[random_mult] Bad disribution.");
        ret = 1;
        break;
      }
      return Math.Max(ret, 1);
    }

    [ConsoleCommand(name: "randmult", docs: "List items.")]
    public static void randmult(string[] args)
    {
      string cmd = args.Length < 1 ? "" : args[0].ToLower();
      string msg;

      if (cmd == "list")
      {
        Regex re = null;
        if (args.Length > 2)
        {
          Debug.Log("usage: randmult list [regex]");
          return;
        }
        else if (args.Length == 2)
        {
          re = new Regex(args[1],
            RegexOptions.IgnoreCase | RegexOptions.Compiled);
        }

        var list = new System.Collections.Generic.SortedSet<string>();
        foreach (var item in ItemManager.GetAllItems())
        {
          var term = item.settings_Inventory?.LocalizationTerm;
          if (term is object && term.Length > 0)
            list.Add(term);
        }

        foreach (var item in list)
        {
          if (re is null || re.IsMatch(item))
            Debug.Log(item);
        }
      }
      else if (cmd == "settings")
      {
        msg = $@"settings:";
        foreach (var pair in multiplier_names)
          msg += $"\n  {pair.Key} {multiply(1 << pair.Value)}";
        Debug.Log(msg);
      }
      else
      {
          Debug.Log(@"usage: randmult command [args ...]

commands
list [regex]
  List items optionally filtered by a regular expression.
settings
  List current settings.");
      }
    }
  }
}